//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/SigColNoise.c,v $
// $Id: SigColNoise.c,v 1.1 2002/12/15 16:42:44 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

const char sweep_CVSID[] = "$Id: SigColNoise.c,v 1.1 2002/12/15 16:42:44 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)
#include "SigPinkNoise.h"

#include <math.h>
#include <drfftw.h> 
USING_NAMESPACE(std)

#include <AH/Etech/Types.h> // always the first include
#include <AH/Error/Sys.h>
#include <AH/Error/Intern.h>
#include <AH/Math/Ft/Wisdom.h>
USING_NAMESPACE(AH)

#include "SigWhiteNoise.h"
#include "Util.h"

// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

SignalColoredNoise::SignalColoredNoise
(
    double amplitude,
    unsigned int samplerate, 
    unsigned int debug
)
    : SignalBase(amplitude, samplerate, debug)
{
}

void SignalColoredNoise::addSignal
(
    double* data,
    unsigned int num
)
{
    if (debon) cerr << "SPN::addSignal: started" << endl;
    if (!data) throw ErrorInternNullPtrArg("SignalColoredNoise::addSignal");
    if (!num) throw ErrorInternBadArg("SignalColoredNoise::addSignal");

    fftw_real* td = new fftw_real[num];      // input samples
    if (!td) throw ErrorSysOutOfMem("SignalColoredNoise::addSignal");
    for (unsigned int i = 0; i < num; ++i)
	td[i] = 0;

    fftw_real* sdata = new fftw_real[num];    // output samples
    if (!sdata)
    {
	delete[] td;
	throw ErrorSysOutOfMem("SignalColoredNoise::addSignal");
    }

    // fill td with white noise:
    SignalWhiteNoise white_noise(maxAmpl, debon);
    white_noise.addSignal(td, num);

    // get plan for FFT:
    MathFtWisdom wis(debon);
    if (debon) cerr << "SPN::addSignal: get plan for " << num
		    << " points" << endl;
    rfftw_plan p = wis.getPlan(num, FFTW_FORWARD);

    // run FFT:
    if (debon) cerr << "SPN::addSignal: run FFT" << endl;
    rfftw_one(p, td, sdata);
    for (unsigned int i = 0; i < num; ++i)
	sdata[i] /= num;

    // convert white noise spectrum to colored noise:
    this->shapeNoiseSpectrum(sdata, num);

    // get plan for inverse FFT:
    if (debon) cerr << "SPN::addSignal: get plan for " << num
		    << " points" << endl;
    p = wis.getPlan(num, FFTW_BACKWARD);

    // run inverse FFT:
    if (debon) cerr << "SPN::addSignal: run inverse FFT" << endl;
    for (unsigned int i = 0; i < num; ++i)
	td[i] = 0;
    rfftw_one(p, sdata, td);
    delete[] sdata; sdata = 0;
    td[0] = 0;

    // correct amplitude:
    if (debon) cerr << "SPN::addSignal: correct amplitude" << endl;
    Util::normalize(td, num, maxAmpl);

    // copy td into data:
    if (debon > 1) cerr << "SPN::addSignal: add to data" << endl;
    for (unsigned int i = 0; i < num; ++i)
	data[i] += td[i];
    delete[] td;
}


void SignalColoredNoise::shapeNoiseSpectrum
(
    double* sdata,
    unsigned int num
)
{
    // convert white noise spectrum to colored noise:
    sdata[0] *= getRealFactor(0);
    unsigned int snum = (num+1)/2;
    for (unsigned int i = 1; i < snum; ++i)
    {
	sdata[i] *= getRealFactor(i); // = real[i]
	sdata[num-i] *= getImagFactor(i); // = imag[i]
    }
    if (num % 2)
    {
	sdata[num/2] *= getRealFactor(num/2); // = real[num/2]
    }
}

// --- end of file ---
