//---------------------------------------------------------------------------
// Copyright 2002 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/gensig/Signals.h,v $
// $Id: Signals.h,v 1.9 2005/10/29 10:44:04 ah Exp $ 
//
// Description: signal factory
//
// How to add a new signal
// -----------------------
// 1. declare Signals::get<type>() member function
// 2. See Signals.c
//
//---------------------------------------------------------------------------

#ifndef SIGNALS_H
#define SIGNALS_H

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <vector>
#include <string>

// --- forward declarations ---

class SignalBase;

// --- types and classes ---

class Signals
{
 public:
    Signals(unsigned int samplerate, unsigned int samplebits, double tim,
	    unsigned int debug = 0);
    ~Signals();
    void add(const std::string& command);
    void calculate(double* data, unsigned int num);
    double getMaxAmplitude() const { return maxAmp; }
    static std::string help(const std::string& header);
    static std::string mlsTable(const std::string& header);
    
 private:
    SignalBase* getDc(const std::vector<std::string>& arg);
    SignalBase* getDirac(const std::vector<std::string>& arg);
    SignalBase* getSinus(const std::vector<std::string>& arg);
    SignalBase* getSquare(const std::vector<std::string>& arg);
    SignalBase* getSaw(const std::vector<std::string>& arg);
    SignalBase* getTriangle(const std::vector<std::string>& arg);
    SignalBase* getSweep(const std::vector<std::string>& arg);
    SignalBase* getRiaaSweep(const std::vector<std::string>& arg);
    SignalBase* getRandomSweep(const std::vector<std::string>& arg);
    SignalBase* getWhiteNoise(const std::vector<std::string>& arg);
    SignalBase* getWhiteNoiseRange(const std::vector<std::string>& arg);
    SignalBase* getPinkNoise(const std::vector<std::string>& arg);
    SignalBase* getPinkNoiseRange(const std::vector<std::string>& arg);
    SignalBase* getMsb(const std::vector<std::string>& arg);
    SignalBase* getLsb(const std::vector<std::string>& arg);
    SignalBase* getMls(const std::vector<std::string>& arg);
    SignalBase* getSilence(const std::vector<std::string>& arg);

    unsigned int sr;
    unsigned int sb;
    double maxAmp;
    double t;
    unsigned int debon;
    std::vector<SignalBase*> signals;
};

#endif // !SIGNALS_H
// --- end of file ---
