#include <stdio.h>

int gen_mls (int* sequence, size_t size, unsigned int order, unsigned int
polynom,
             unsigned int* length_generated);

int test_mls (int* sequence, size_t size, unsigned int length, unsigned
int* peak_count,
              unsigned int* minus_one_count, unsigned int* other_count);

/*#define MAX_LENGTH 16777215
#define POLY 0x00e10000
#define ORDER 24*/

#define MAX_LENGTH 32767
#define POLY 0x00004001
#define ORDER 15

/*#define MAX_LENGTH 65535
#define POLY 0x00000012
#define ORDER 5*/

int main(int argc, char* argv[])
{
    static int sequence[MAX_LENGTH];
    unsigned int length, peak_count, minus_one_count, other_count;

    gen_mls (sequence, MAX_LENGTH, ORDER, POLY, &length);
    test_mls (sequence, MAX_LENGTH, length, &peak_count, &minus_one_count,
&other_count);
    if (peak_count == 1 && minus_one_count == length -1 && other_count == 0)
        printf ("Sequence is a Maximum Length Sequence.\n\n");
    else
        printf ("Sequence is NOT a Maximum Length Sequence!\n\n");
    return 0;
}

#if 0
int test_mls (int* sequence, size_t size, unsigned int length, unsigned
int* peak_count,
              unsigned int* minus_one_count, unsigned int* other_count)
{
    unsigned int i, j, k;
    int auto_corr;

    *peak_count = 0;
    *minus_one_count = 0;
    *other_count = 0;
    for (i = 0; i < length; i++)
    {
        /* Calculate _one_ value of auto-correlation */
        auto_corr = 0;
        for (j = 0; j < length; j++)
        {
#if 1
	    k = (i + j) % length;
#else
            /* Poor man's modulo */
            if (i + j < length)
                k = i + j;
            else
                k = i + j - length;
#endif

            auto_corr += sequence [j] * sequence [k];
        }
        /* Count the occurence of the following values:
            * length
            * -1
            * other values
        */
        if (auto_corr == -1)
            (*minus_one_count)++;
        else if (auto_corr == length)
            (*peak_count)++;
        else
            (*other_count)++;
    }
    return 0;
}
#else
int test_mls (int* sequence, size_t size, unsigned int length, unsigned
int* peak_count,
              unsigned int* minus_one_count, unsigned int* other_count)
{
    unsigned int i, j, k, l;
    int auto_corr;

    *peak_count = 0;
    *minus_one_count = 0;
    *other_count = 0;
    for (i = 0; i < length; i++)
    {
        /* Calculate _one_ value of auto-correlation */
        auto_corr = 0;
        l = length - i;
        k = i;
        for (j = 0; j < l; j++)
        {
            auto_corr += sequence [j] * sequence [k++];
        }
        if (i > 0)
        {
            k = k - length;
            for (; j < length; j++)
            {
                auto_corr += sequence [j] * sequence [k++];
            }
        }
        /* Count the occurence of the following values:
            * length
            * -1
            * other values
        */
        if (auto_corr == -1)
            (*minus_one_count)++;
        else if (auto_corr == length)
            (*peak_count)++;
        else
            (*other_count)++;
    }
    return 0;
}
#endif

int gen_mls (int* sequence, size_t size, unsigned int order, unsigned int
polynom,
             unsigned int* length_generated)
{
    unsigned int xor, one_bit_pattern = 1, length = 0, shift_reg = 0xFFFFFFFF,
                 num_of_taps = 0, end_of_shift_reg = 1, i, j;
    unsigned int tap[32];

    *length_generated = 0;
    if (order < 2 || order > 32)
        return 1;
    /* create table of taps */
    for (i = 0; i < order; i++)
    {
        if (one_bit_pattern & polynom)
            tap[num_of_taps++] = one_bit_pattern;
        length = length + one_bit_pattern;
        one_bit_pattern = one_bit_pattern << 1;
    }
    if (length > size)
        return 1;
    for (i=1; i < order; i++)
        end_of_shift_reg = end_of_shift_reg << 1;
    /* generate mls */
    for (i = 0; i < length; i++)
    {
        xor = 0;
        for (j = 0; j < num_of_taps; j++)
            if (tap[j] & shift_reg) xor = xor ^ 1;
        sequence[i] = (shift_reg & end_of_shift_reg) ? 1 : -1;
        shift_reg = (shift_reg << 1) | xor;
    }
    *length_generated = i;
    return 0;
}
