//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Current.h,v $
// $Id: Current.h,v 1.6 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_CURRENT_H	// prevent multiple includes
#define ETECH_CURRENT_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/PhysUnit.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an electrical current
/**
 {\tt Current} represents an electrical current which can be used both in
 DC (direct current) and in AC (alternate current) applications.
 */
class Current: public PhysUnitCplx
{
  public:
    /// constructor
    Current(Dbl r = 0.0, Dbl i = 0.0) : PhysUnitCplx("A",r,i) {}
    /// constructor
    Current(const Cplx &val) : PhysUnitCplx("A",val) {}
    /// copy constructor
    Current(const Current& i) : PhysUnitCplx("A",i.Value) {}

    /// overloaded operator + for addition of two currents
    friend Current operator + (const Current &i1, const Current &i2)
	{ return Current(i1.Value + i2.Value); }
    /// overloaded operator - for subtraction of two currents
    friend Current operator - (const Current &i1, const Current &i2)
	{ return Current(i1.Value - i2.Value); }

    /// overloaded operator * for multiplication of {\tt Current} * {\tt Cplx}
    friend Current operator * (const Current &i1, const Cplx &k)
	{ return Current(i1.Value * k); }
    /// overloaded operator * for multiplication of {\tt Cplx} * {\tt Current}
    friend Current operator * (const Cplx &k, const Current &i1)
	{ return Current(k * i1.Value); }

    /// overloaded operator / for division of {\tt Current} / {\tt Cplx}
    friend Current operator / (const Current &i1, const Cplx &k)
	{ return Current(i1.Value / k); }

    /// overloaded operator == for comparison of two Currents
    friend bool operator == (const Current &i1, const Current& i2)
	{ return (i1.Value == i2.Value); }
    /// overloaded operator != for comparison of two Currents
    friend bool operator != (const Current &i1, const Current& i2)
	{ return (i1.Value != i2.Value); }
    /// overloaded operator < for comparison of two Currents
    friend bool operator < (const Current &i1, const Current& i2)
	{ return (i1.Value < i2.Value); }
    /// overloaded operator > for comparison of two Currents
    friend bool operator > (const Current &i1, const Current& i2)
	{ return (i1.Value > i2.Value); }

};


END_NAMESPACE

#endif // ETECH_CURRENT_H
