//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Filter2PoleInv.h,v $
// $Id: Filter2PoleInv.h,v 1.3 2001/10/05 23:44:27 ah Exp $
//
// Description: class Filter2PoleInv
//
//---------------------------------------------------------------------------

#ifndef ETECH_FILTER2POLEINV_H
#define ETECH_FILTER2POLEINV_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/OpAmp.h>
#include <AH/Etech/Amplifier.h>

USING_NAMESPACE(Ah)
BEGIN_NAMESPACE(Ah)

// --- forward declarations ---

class Resistor;
class Capacitor;

// --- types and classes ---


// --- types and classes ---

/// 2-pole filter using multiple feedback on inverting amplifier
/**
 {\tt Filter2PoleInv} is a 2 pole filter realized with an inverting 
 (operational) amplifier having multiple feedback paths.
 \begin{verbatim}
            _________________
            |      |        |
            Z2     Z5       |
            |      |  |-\   |
  In: --Z1--*--Z24-*--|- \  |
            |         |   ----- Out
            Z3      --|+ /
            |       | |_/
           Gnd     Gnd
 \end{verbatim}
 Typical applications are as follows:
 \begin{description}
 \item[Lowpass:] Z1, Z2 and Z4 are resistors, Z3 and Z5 are capacitors, e.g:\\
	Z1=R1, Z2=R2, Z3=C2, Z4=R3, Z5=C1\\
	$R_1=-R_2/A_0$
 \item[Highpass:] Z3 and Z5 are resistors, Z1, Z2 and Z4 are capacitors.
 \item[Bandpass:] Z1, Z3 and Z5 are resistors, Z2 and Z4 are capacitors.
 \end{description}
 */
class Filter2PoleInv: public Amplifier
{
  public:
//    enum type {Lowpass, Bandpass, Highpass, Unknown};

    /// constructors:
    Filter2PoleInv(const OpAmp& opamp, const Impedance& z1, 
		   const Impedance& z2, const Impedance& z3, 
		   const Impedance& z4, const Impedance& z5);
//    Filter2PoleInv(const OpAmp& opamp, type t, Dbl fg, Dbl a1, Dbl a2);

    /// destructor:
    virtual ~Filter2PoleInv() {}

    /// get amplification at frequency {\tt freq}:
    virtual Cplx getA(Dbl freq);
    /// get amplification at frequency {\tt freq} with output {\tt load}:
    virtual Cplx getA(Dbl freq, const Impedance& )
	{ return getA(freq); }

    /// calculate components
    static bool calcLowpass
	(Dbl fg, Dbl A0, Dbl a1, Dbl b1, 
	 const Capacitor& C1, const Capacitor& C2, 
	 Resistor& R1, Resistor& R2, Resistor& R3);

  private:
    OpAmp     op;	// used operational amplifier
    Impedance Z1;	// feedback network
    Impedance Z2;	// feedback network
    Impedance Z3;	// feedback network
    Impedance Z4;	// feedback network
    Impedance Z5;	// feedback network
//    type      filter;	// type of filter
};

// --- name space ---

END_NAMESPACE

#endif /* !ETECH_FILTER2POLEINV_H */
