//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Inductor.h,v $
// $Id: Inductor.h,v 1.8 2004/04/20 22:04:48 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_INDUCTOR_H
#define ETECH_INDUCTOR_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/FreqImp.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an ideal electrical inductor
/**
 {\tt Inductor} represents an ideal electrical inductor which can be used both 
 in DC (direct current) and in AC (alternate current) applications.
 */
class Inductor: public FreqImpedance
{
  private:
    Dbl Inductivity;

  public:
    /// constructor
    Inductor(Dbl l = 0.0) : 
	FreqImpedance(), Inductivity(l), frequency(-1) {}
//    /// constructor
//    Inductor(const FreqImpedance& i) : FreqImpedance(i), frequency(-1) {}
    /// copy constructor
    Inductor(const Inductor& l) 
	: FreqImpedance(l.Value), Inductivity(l.Inductivity), frequency(-1) {}

    /// return inductivity
    Dbl getInd() const { return Inductivity; }

    /// overloaded operator + for serial circuit of two inductors
    friend Inductor operator + (const Inductor &l1, const Inductor &l2)
	{ return Inductor(l1.Inductivity + l2.Inductivity); }

    /// overloaded operator || for parallel circuit of two inductors
    friend Inductor operator || (const Inductor &l1, const Inductor &l2)
	{ return Inductor(l1.Inductivity * l2.Inductivity / 
			  (l1.Inductivity + l2.Inductivity)); }

    /// overloaded operator * for multiplication of {\tt Inductor} * {\tt Dbl}
    friend Inductor operator * (const Inductor &l, Dbl k)
	{ return Inductor(l.Inductivity * k); }
    /// overloaded operator * for multiplication of{\t Dbl} * {\tt Inductor}
    friend Inductor operator * (Dbl k, const Inductor &l)
	{ return Inductor(k * l.Inductivity); }

    /// overloaded operator / for division of {\tt Inductor} / {\tt Dbl}
    friend Inductor operator / (const Inductor &l, Dbl k)
	{ return Inductor(l.Inductivity / k); }

    /// overloaded operator for output into streams
    friend std::ostream& operator << (std::ostream &out, const Inductor &i);

  protected:
    virtual void calcVal(Dbl f);

  private:
    Dbl frequency;	// just for caching
}; // class Inductor


END_NAMESPACE

#endif // ETECH_INDUCTOR_H
