//---------------------------------------------------------------------------
// Copyright 2004 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Math/DataConv.h,v $
// $Id: DataConv.h,v 1.2 2004/04/06 23:14:43 ah Exp $ 
//
// Description: Interface to data convert functions
//
//---------------------------------------------------------------------------

#ifndef AH_MATH_DATA_CONV_H
#define AH_MATH_DATA_CONV_H

// --- necessary includes ---

#include <AH/Config/Platform.h>

// --- types and classes ---

BEGIN_NAMESPACE(AH)


class DataConv
{
 public:
    /// reduce data linear
    /**
       DataConv::reduceLinear() takes <i>dataNum</i> double values from
       <i>data</i> and reduces them to <i>outNum</i> double values in
       one or more of <i>outMean</i>, <i>outMax</i> and <i>outMin</i>,
       if these are not NULL pointers. 
       Usually <i>outNum></i> is smaller than <i>dataNum</i>, so 
       <ul>
         <li> <i>outMean</i> contains the mean value of those data points
	      in <i>data</i> which correspond to one point in <i>outMean</i>.
         <li> <i>outMax</i> contains the maximum value of those data points
	      in <i>data</i> which correspond to one point in <i>outMax</i>.
         <li> <i>outMin</i> contains the minimum value of those data points
	      in <i>data</i> which correspond to one point in <i>outMin</i>.
       </ul>
       If <i>outNum></i> is <b>not</b>smaller than <i>dataNum</i>, then
       <i>outMean</i>, <i>outMax</i> and <i>outMin</i> contain the same 
       values.
     */
    static void reduceLinear
    (
	/// input data
	const double* data,
	/// number of input data
	unsigned int dataNum,
	/// number of output data
	unsigned int outNum,
	/// output data for mean value, may be 0 if not needed
	double* outMean,
	/// output data for maximum value, may be 0 if not needed
	double* outMax,
	/// output data for minimum value, may be 0 if not needed
	double* outMin
    );

 private:
};


// --- function prototypes ---

END_NAMESPACE

#endif // !AH_MATH_DATA_CONV_H
// --- end of file ---
