//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/VierPol.c,v $
// $Id: VierPol.c,v 1.4 2004/04/20 22:14:57 ah Exp $
//
// Description: implementation of vierpol classes
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: VierPol.c,v 1.4 2004/04/20 22:14:57 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/VierPol.h>
#include <iostream>

USING_NAMESPACE(Ah)

BEGIN_NAMESPACE(Ah)

// --- local stuff ---

// --- class member implementation ---

//-------------------------------------------------------------------
// class VierPol :
//-------------------------------------------------------------------

std::ostream& operator << (std::ostream &out, const VierPol &vp)
{
    if (true /*out.opfx()*/)
    {
	if (vp.isValid())
	    out << vp.c11 << "  " << vp.c12 << "\n" 
		<< vp.c21 << "  " << vp.c22;
	else 
	    out << "invalid";
    }
    return out;
}

//-------------------------------------------------------------------
// class VierPolZ :
//-------------------------------------------------------------------

// convert constructor
VierPolZ::VierPolZ (const VierPolY& vpy)
    : VierPol(vpy.y22() / vpy.det(),  -vpy.y12() / vpy.det(),
	      -vpy.y21() / vpy.det(), vpy.y11() / vpy.det())
{
}


// convert constructor
VierPolZ::VierPolZ (const VierPolH& vph)
    : VierPol(vph.det() / vph.h22(),   vph.h12() / vph.h22(),
	      -vph.h21() / vph.h22(),  1 / vph.h22())
{
}


// convert constructor
VierPolZ::VierPolZ (const VierPolA& vpa)
    : VierPol(vpa.a11() / vpa.a21(),  vpa.det() / vpa.a21(),
	      1 / vpa.a21(),	      vpa.a22() / vpa.a21())
{
}

// --- overloaded operators ---

// convert VierPolY into VierPolZ
const VierPolZ& VierPolZ::operator = (const VierPolY& vpy)
{
    Cplx det(vpy.det());
    Cplx null(0,0);
    if (det != null)
    {
	c11 = vpy.y22() / det;
	c12 = -vpy.y12() / det;
	c21 = -vpy.y21() / det;
	c22 = vpy.y11() / det;
    }
    else
    {
	Cplx inf(0,0, Cplx::Inf);
	c11 = c12 = c21 = c22 = inf;
    } 
    return *this;
}

// convert VierPolH into VierPolZ
const VierPolZ& VierPolZ::operator = (const VierPolH& vph)
{
    c11 = vph.det() / vph.h22();
    c12 = vph.h12() / vph.h22();
    c21 = -vph.h21() / vph.h22();
    c22 = 1 / vph.h22();
    return *this;
}

// convert VierPolA into VierPolZ
const VierPolZ& VierPolZ::operator = (const VierPolA& vpa)
{
    c11 = vpa.a11() / vpa.a21();
    c12 = vpa.det() / vpa.a21();
    c21 = 1 / vpa.a21();
    c22 = vpa.a22() / vpa.a21();
    return *this;
}

//-------------------------------------------------------------------
// class VierPolY :
//-------------------------------------------------------------------

// convert constructor
VierPolY::VierPolY (const VierPolZ& vpz)
    : VierPol()
{
}

// convert constructor
VierPolY::VierPolY (const VierPolH& vph)
    : VierPol()
{
}

// convert constructor
VierPolY::VierPolY (const VierPolA& vpa)
    : VierPol()
{
}

// convert VierPolZ into VierPolY
const VierPolY& VierPolY::operator = (const VierPolZ& vpz)
{
    Cplx det(vpz.det());
    Cplx null(0,0);
    if (det != null)
    {
	c11 = vpz.z22() / det;
	c12 = -vpz.z12() / det;
	c21 = -vpz.z21() / det;
	c22 = vpz.z11() / det;
    }
    else
    {
	Cplx inf(0,0, Cplx::Inf);
	c11 = c12 = c21 = c22 = inf;
    } 
    return *this;
}

// convert VierPolH into VierPolY
const VierPolY& VierPolY::operator = (const VierPolH& vph)
{
    return *this;
}

// convert VierPolA into VierPolY
const VierPolY&VierPolY:: operator = (const VierPolA& vpa)
{
    return *this;
}

 //-------------------------------------------------------------------
// class VierPolH :
//-------------------------------------------------------------------

// convert constructor
VierPolH::VierPolH (const VierPolY& vpy)
    : VierPol(1 / vpy.y11(),		-vpy.y12() / vpy.y11(),
	      vpy.y21() / vpy.y11(),	vpy.det() / vpy.y11())
{
}

// convert constructor
VierPolH::VierPolH (const VierPolZ& vpz)
    : VierPol()
{
}

// convert constructor
VierPolH::VierPolH (const VierPolA& vpa)
    : VierPol()
{
}

// convert VierPolZ into VierPolH
const VierPolH& VierPolH::operator = (const VierPolZ& vpz)
{
    return *this;
}

// convert VierPolH into VierPolH
const VierPolH& VierPolH::operator = (const VierPolY& vpy)
{
    return *this;
}

// convert VierPolA into VierPolH
const VierPolH& VierPolH::operator = (const VierPolA& vpa)
{
    return *this;
}

//-------------------------------------------------------------------
// class VierPolA :
//-------------------------------------------------------------------

// convert constructor
VierPolA::VierPolA (const VierPolY& vpy)
    : VierPol(-vpy.y22() / vpy.y21(),	-1 / vpy.y21(),
	      -vpy.det() / vpy.y21(),	-vpy.y11() / vpy.y21())
{
}

// convert constructor
VierPolA::VierPolA (const VierPolH& vph)
    : VierPol()
{
}
// convert constructor
VierPolA::VierPolA (const VierPolZ& vpz)
    : VierPol()
{
}

// convert VierPolZ into VierPolA
const VierPolA& VierPolA::operator = (const VierPolZ& vpz)
{
    return *this;
}

// convert VierPolH into VierPolA
const VierPolA& VierPolA::operator = (const VierPolH& vph)
{
    return *this;
}

// convert VierPolY into VierPolA
const VierPolA& VierPolA::operator = (const VierPolY& vpy)
{
    return *this;
}


END_NAMESPACE
