//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestConvert.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestConvert.c,v 1.4 2005/09/21 23:54:48 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/Convert.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <sstream>

USING_NAMESPACE(std)

#include <AH/Test/Log.h>

// --- local implementation ---

static bool debug = false;

//----------------------------------------------------------------------
// test data for integer arguments:
//----------------------------------------------------------------------

static struct TestIntData
{
    unsigned int value;
    const char* unit;
    const char* result;
} testIntData[] = 
{
    // testing just seconds:
    {  8, "s", "00:00:08" },
    { 40, "s", "00:00:40" },

    // testing just minutes + seconds:
    {  1*60 +  7, "s", "00:01:07" },
    {  1*60 + 20, "s", "00:01:20" },
    { 15*60 +  5, "s", "00:15:05" },
    { 15*60 +  0, "s", "00:15:00" },

    // testing just hours, minutes + seconds:
    {  1*60*60 +  1*60 +  7 , "s", "01:01:07" },
    { 15*60*60 +  1*60 + 20, "s", "15:01:20" },
    { 23*60*60 + 15*60 +  5, "s", "23:15:05" },
    { 22*60*60 +  0*60 +  5, "s", "22:00:05" },
    { 21*60*60 +  0*60 +  0, "s", "21:00:00" },

    // testing just days:
    {  1*24*60*60 +  0*60*60 +  0*60 +  0, "s", "1 day" },
    { 28*24*60*60 +  0*60*60 +  0*60 +  0, "s", "28 days" },
    // testing days, hours, minutes + seconds:
    { 1*24*60*60 +  1*60*60 +  1*60 +  7 , "s", "1 day 01:01:07" },
    { 28*24*60*60 + 15*60*60 +  1*60 + 20, "s", "28 days 15:01:20" },

    // testing just months:
    {  1*30*24*60*60, "s", "1 month" },
    { 11*30*24*60*60, "s", "11 months" },
    // testing just months and days:
    {  1*30*24*60*60 +  1*24*60*60, "s", "1 month 1 day" },
    { 11*30*24*60*60 + 28*24*60*60, "s", "11 months 28 days" },
    // testing months, days, hours, minutes + seconds:
    {  1*30*24*60*60 +  1*24*60*60 +  1*60*60 +  1*60 + 7, "s", 
      "1 month 1 day 01:01:07" },
    { 11*30*24*60*60 + 28*24*60*60 + 15*60*60 +  1*60 + 20, "s", 
      "11 months 28 days 15:01:20" },

    // testing just years:
    {  1*12*30*24*60*60, "s", "1 year" },
    { 11*12*30*24*60*60, "s", "11 years" },
    // testing just years and months:
    {  1*12*30*24*60*60 +  1*30*24*60*60, "s", "1 year 1 month" },
    { 11*12*30*24*60*60 + 11*30*24*60*60, "s", "11 years 11 months" },
    // testing just years and months and days:
    {  1*12*30*24*60*60 +  1*30*24*60*60 +  1*24*60*60, "s",
       "1 year 1 month 1 day" },
    { 11*12*30*24*60*60 + 11*30*24*60*60 + 28*24*60*60, "s",
      "11 years 11 months 28 days" },
    // testing years, months, days, hours, minutes + seconds:
    {  1*12*30*24*60*60 +  1*30*24*60*60 +  1*24*60*60 +  1*60*60 +  1*60 + 7,
       "s", "1 year 1 month 1 day 01:01:07" },
    { 11*12*30*24*60*60 + 11*30*24*60*60 + 28*24*60*60 + 15*60*60 +  1*60 + 20,
      "s", "11 years 11 months 28 days 15:01:20" }

};

static struct TestDoubleData
{
    double value;
    const char* unit;
    const char* result;
} testDoubleData[] = 
{
    // testing just seconds:
    {  8 + 0.1, "s", "00:00:08.100" },
    { 40 + 0.001234, "s", "00:00:40.001234" },
    { 40 + 0.0012345, "s", "00:00:40.001234500" },

    // testing just minutes + seconds:
    {  1*60 +  7, "s", "00:01:07" },
    {  1*60 + 20, "s", "00:01:20" },
    { 15*60 +  5, "s", "00:15:05" },
    { 15*60 +  0, "s", "00:15:00" },

    // testing just hours, minutes + seconds:
    {  1*60*60 +  1*60 +  7 , "s", "01:01:07" },
    { 15*60*60 +  1*60 + 20, "s", "15:01:20" },
    { 23*60*60 + 15*60 +  5, "s", "23:15:05" },
    { 22*60*60 +  0*60 +  5, "s", "22:00:05" },
    { 21*60*60 +  0*60 +  0, "s", "21:00:00" },

    // testing just days:
    {  1*24*60*60 +  0*60*60 +  0*60 +  0, "s", "1 day" },
    { 28*24*60*60 +  0*60*60 +  0*60 +  0, "s", "28 days" },
    // testing days, hours, minutes + seconds:
    { 1*24*60*60 +  1*60*60 +  1*60 +  7 , "s", "1 day 01:01:07" },
    { 28*24*60*60 + 15*60*60 +  1*60 + 20, "s", "28 days 15:01:20" },

    // testing just months:
    {  1*30*24*60*60, "s", "1 month" },
    { 11*30*24*60*60, "s", "11 months" },
    // testing just months and days:
    {  1*30*24*60*60 +  1*24*60*60, "s", "1 month 1 day" },
    { 11*30*24*60*60 + 28*24*60*60, "s", "11 months 28 days" },
    // testing months, days, hours, minutes + seconds:
    {  1*30*24*60*60 +  1*24*60*60 +  1*60*60 +  1*60 + 7, "s", 
      "1 month 1 day 01:01:07" },
    { 11*30*24*60*60 + 28*24*60*60 + 15*60*60 +  1*60 + 20, "s", 
      "11 months 28 days 15:01:20" },

    // testing just years:
    {  1*12*30*24*60*60, "s", "1 year" },
    { 11*12*30*24*60*60, "s", "11 years" },
    // testing just years and months:
    {  1*12*30*24*60*60 +  1*30*24*60*60, "s", "1 year 1 month" },
    { 11*12*30*24*60*60 + 11*30*24*60*60, "s", "11 years 11 months" },
    // testing just years and months and days:
    {  1*12*30*24*60*60 +  1*30*24*60*60 +  1*24*60*60, "s",
       "1 year 1 month 1 day" },
    { 11*12*30*24*60*60 + 11*30*24*60*60 + 28*24*60*60, "s",
      "11 years 11 months 28 days" },
    // testing years, months, days, hours, minutes + seconds:
    {  1*12*30*24*60*60 +  1*30*24*60*60 +  1*24*60*60 +  1*60*60 +  1*60 + 7,
       "s", "1 year 1 month 1 day 01:01:07" },
    { 11*12*30*24*60*60 + 11*30*24*60*60 + 28*24*60*60 + 15*60*60 +  1*60 + 20,
      "s", "11 years 11 months 28 days 15:01:20" }

};

//----------------------------------------------------------------------
// --- global implementation ---
//----------------------------------------------------------------------


int main(int argc, char* argv[])
{
    if ((2 <= argc) && (0 == strcmp("-d", argv[1])))
	debug = true;

    TestLog testlog(cout, "function IOTimeFomat", debug);

    //-------------------------
    // test integer function:
    //-------------------------
    testlog.announceSeries("function IOConvert::timeToString(uint)");
    int i;
    int size = sizeof(testIntData)/sizeof(testIntData[0]);
    for (i=0; i < size; i++)
    {
	ostringstream what;
	what << testIntData[i].value << " " << testIntData[i].unit;
	testlog.announceTest(what.str());
	string result = IOConvert::timeToString(testIntData[i].value,
						testIntData[i].unit);
	testlog.returned(result);
	if (result != testIntData[i].result)
	    testlog.failed(testIntData[i].result);
	else 
	    testlog.passed();
    }

    //-------------------------
    // test Double function:
    //-------------------------
    testlog.announceSeries("function IOConvert::timeToString(double)");
    size = sizeof(testDoubleData)/sizeof(testDoubleData[0]);
    for (i=0; i < size; i++)
    {
	ostringstream what;
	what << testDoubleData[i].value << " " << testDoubleData[i].unit;
	testlog.announceTest(what.str());
	string result = IOConvert::timeToString(testDoubleData[i].value,
						testDoubleData[i].unit);
	testlog.returned(result);
	if (result != testDoubleData[i].result)
	    testlog.failed(testDoubleData[i].result);
	else 
	    testlog.passed();
    }

    //----------------
    // print results:
    //----------------
    return testlog.makeReport();
}
