//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestRiffWaveFmt.c,v $
// $Id: TestRiffWaveFmt.c,v 1.2 2004/04/20 22:26:22 ah Exp $
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestRiffWaveFmt.c,v 1.2 2004/04/20 22:26:22 ah Exp $ (c) 1999 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/Riff/WaveFmt.h>	// my interface first, says Lakos
#include <AH/IO/Riff/StartChunk.h>	// my interface first, says Lakos
#include <AH/Error/Class.h>
#include <AH/IO/Smart/FilePtr.h>
#include <AH/IO/Smart/Buffer.h>
USING_NAMESPACE(AH)

#include <cstdlib>
#include <unistd.h>
#include <string>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
USING_NAMESPACE(std)

static unsigned int execute_test
(
    const string& name,
    FILE* fp
)
{
    try
    {
	//-------------------
	// read start chunk:
	//-------------------
	IOSmartBuffer<unsigned char> buf(IORiffStartChunk::size);
	if (0 == fread(buf(), buf.getSize(), 1, fp)) 
	    throw ErrorSys(name);
	IORiffStartChunk startChunk(buf());

	//--------------------
	// read format chunk:
	//--------------------
	buf.resize(IORiffChunk::size);
	if (0 == fread(buf(), buf.getSize(), 1, fp)) 
	    throw ErrorSys(name);
	IORiffChunk formatchunk(buf());

	//--------------------
	// read format data:
	//--------------------
	buf.resize(formatchunk.getDataSize());
	if (0 == fread(buf(), buf.getSize(), 1, fp)) 
	    throw ErrorSys(name);
	IORiffWaveFmt fmt(buf());
	cout << "Format: " << fmt.getFormatKey() << endl;
	cout << "Channels: " << fmt.getChannels() << endl;
	cout << "SampleRate: " << fmt.getSampleRate() << endl;
	cout << "DataRate: " << fmt.getDataRate() << endl;
	cout << "BlockSize: " << fmt.getBlockSize() << endl;

	fmt.printInfo(cout, "fmt: ");
    }
    catch(ErrorClass& error)
    {
	return 1;
    }
    return 0;
}



int main(int argc, char* argv[])
{
    cerr << "Test of " << argv[0] << ":" << endl;

    unsigned int errnum = 0;
    for (int i=1; i < argc; i++)
    {
	string filename(argv[i]);
	cerr << "processing " << filename << " ..." << endl;

	IOSmartFilePtr file(filename, "r");
	errnum += execute_test(filename, file());
    }

    // print test results:
    cerr << argv[0] << ": ";
    if (errnum)
    {
	cerr << "failed with " << errnum << " error"
	     << (errnum > 1 ? "s" : "") << ". stop." << endl;
	return 1;;
    }
    cerr << "passed." << endl;
    return 0;
}

